/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "cache.h"
#include "strings.h"
#include "global.h"
#include "files.h"
#include "lang.h"
#include "strdump.h"
#include "context.h"

#include <common/debug.h>
#include <common/config.h>
#include <common/restart.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>

#define CACHE_VERSION 1

bool cache_ready = false;

static void cache_restart(void *unused)
{
  dprinte("Restarting cache");
  cache_ready = false;
  cache_string_restart();
  cache_string_dump_restart();
  cache_files_restart();
  cache_lang_restart();
  cache_context_restart();
}

void cache_init(void)
{
  cache_global_init();
  cache_string_init();
  cache_string_dump_init();
  cache_files_init();
  cache_lang_init();
  cache_context_init();
  restart_cleanup_register(cache_restart, NULL);
}

void cache_load(const char *filename)
{
  if(nocache_flag)
    return;
  dprinte("Loading cache");
  // TODO Some header to check everything works
  int fd = open(filename, O_RDONLY);
  if(fd == -1)
  {
    // TODO warning we have no cache
  }
  else
  {
    char intro[3];
    cache_read(fd, intro, 3);
    if(strncmp(intro, "REM", 3))
      die("This is not remake cache, or is corrupt (%s)", filename);
    size_t cache_version;
    cache_read(fd, &cache_version, sizeof cache_version);
    if(cache_version != CACHE_VERSION)
    {
      fprintf(stderr, "Wrong cache version, not loading, updating to new version. Specify variables again (%s).\n",
          filename);
      nocache_flag = true;
      return;
    }
    cache_global_free_input();
    cache_files_load(fd);
    cache_lang_load(fd);
    cache_context_load(fd);
    cache_string_load(fd);
    cache_string_dump_load(fd);
    if(close(fd))
      die("Could not close cache file (%s)", strerror(errno));
    cache_lang_resolve();
    cache_context_resolve();
    cache_global_free_input_tmp();
    cache_ready = true;
  }
}

void cache_store(const char *filename)
{
  dprinte("Storing cache");
  // TODO Some header to check everything works
  // TODO trough tempfile so the original is not lost on failure?
  int fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC,
      S_IREAD | S_IWRITE);
  if(fd == -1)
    die("Could not write cache (%s)", strerror(errno));
  cache_write(fd, "REM", 3);
  size_t cache_version = CACHE_VERSION;
  cache_write(fd, &cache_version, sizeof cache_version);
  cache_files_store(fd);
  cache_lang_store(fd);
  cache_context_store(fd);
  cache_string_store_drop(fd);
  cache_string_dump_store_drop(fd);
  if(close(fd))
    die("Could not close cache file (%s)", strerror(errno));
  cache_lang_out_free();
  cache_global_free_output();
}
