/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "global.h"

#include <common/debug.h>

#include <string.h>
#include <errno.h>
#include <unistd.h>

struct mempool_core cache_strpool_output;
struct mempool_core cache_pool_output, cache_pool_input, cache_pool_input_tmp;

void cache_global_init(void)
{
  strpool_init(&cache_strpool_output);
  mempool_core_init(&cache_pool_output, 0, true);
  mempool_core_init(&cache_pool_input, 0, true);
  mempool_core_init(&cache_pool_input_tmp, 0, true);
}

// TODO replace die with exception
void cache_read(int fd, void *buf, size_t count)
{
  ssize_t result;
  const unsigned char *pos = buf;
  while((result = read(fd, buf, count)) < (ssize_t) count && result > 0)
  {
    count -= result;
    pos += result;
  }
  if(result == -1)
    die("Error reading cache data (%s)", strerror(errno));
  if(!result && count)
    die("Short read from cache (%u written, %u remains)", (unsigned) (pos - (const unsigned char *) buf),
        (unsigned) count);
}

void cache_write(int fd, const void *buf, size_t count)
{
  ssize_t result;
  const unsigned char *pos = buf;
  while((result = write(fd, pos, count)) < (ssize_t) count && result > 0)
  {
    count -= result;
    pos += result;
  }
  if(result == -1)
    die("Error writing cache (%s)", strerror(errno));
  if(!result && count)
    die("Short write into cache (%u written, %u remains)", (unsigned) (pos - (const unsigned char *) buf),
        (unsigned) count);
}

void cache_global_free_input(void)
{
  mempool_core_clean(&cache_pool_input);
}

void cache_global_free_output(void)
{
  mempool_core_clean(&cache_pool_output);
  mempool_core_clean(&cache_strpool_output);
}

void cache_global_free_input_tmp(void)
{
  mempool_core_clean(&cache_pool_input_tmp);
}
