/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "strdump.h"
#include "global.h"

#include <common/debug.h>

struct strptr
{
  const char *ptr;
  unsigned num;
};

#define STACK_PREFIX(X) ptrs_##X
#define STACK_TYPE struct strptr
#include <common/stack.h>

static struct ptrs_stack ptrs;

void cache_string_dump_init(void)
{
  ptrs_init(&ptrs);
}

void cache_string_dump_store_drop(int fd)
{
  cache_write(fd, &ptrs.size, sizeof ptrs.size);
  cache_write(fd, ptrs.data, ptrs.size * sizeof *ptrs.data);
  ptrs_free(&ptrs);
}

static size_t size;
static struct strptr *data;

void cache_string_dump_load(int fd)
{
  cache_read(fd, &size, sizeof size);
  data = mempool_alloc(&cache_pool_input, size * sizeof *data);
  cache_read(fd, data, size * sizeof *data);
}

const char *cache_string_dump_push(const char *str)
{
  if(!str)
    return NULL;
  for(size_t i = 0; i < ptrs.size; i ++)
    if(!strcmp(ptrs.data[i].ptr, str))
      return ptrs.data[i].ptr;
  ptrs_push(&ptrs, (struct strptr) {
    .ptr = str,
    .num = cache_string_push(str)
  });
  return str;
}

const char *cache_string_dump_get(const char *str)
{
  if(!str)
    return NULL;
  for(size_t i = 0; i < size; i ++)
    if(data[i].ptr == str)
      return cache_string_input.strings[data[i].num];
  diee("Cache corruption");
}

void cache_string_dump_restart(void)
{
  ptrs_free(&ptrs);
  ptrs = (struct ptrs_stack) {};
}
