/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "debug.h"
#include "config.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#ifdef GDB_CATCH
void gdb_catch(int sig)
{
  (void) sig;
  pid_t pid = getpid();
  char buffer[1024];
  snprintf(buffer, 1024, "/tmp/debug-%u", pid);
  FILE *f = fopen(buffer, "wt");
  fprintf(f, "attach %u\nbt\n", pid);
  fclose(f);
  snprintf(buffer, 1024, "gdb -x /tmp/debug-%u", pid);
  system(buffer);
}
#endif

void die_internal(int debug, const char *format, ...)
{
  va_list args;
  va_start(args, format);
  vfprintf(stderr, format, args);
  va_end(args);
  if(debug)
  {
#ifdef GDB_CATCH
    gdb_catch(0);
#endif
    abort();
  }
  exit(1);
}

void dprintf_internal(const char *format, ...)
{
  if(!config_debug)
    return;
  va_list args;
  va_start(args, format);
  vfprintf(stderr, format, args);
  va_end(args);
}

void gdb_catch_signals(void)
{
#ifdef GDB_CATCH
  signal(SIGPIPE, &gdb_catch);
  signal(SIGABRT, &gdb_catch);
  signal(SIGFPE, &gdb_catch);
  signal(SIGILL, &gdb_catch);
  signal(SIGSEGV, &gdb_catch);
  signal(SIGTRAP, &gdb_catch);
#endif
}
