/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "dir.h"

#include "debug.h"
#include "memory.h"

#include <unistd.h>
#include <string.h>
#include <errno.h>

#define DIR_LEN 128

void dir_set(const char *path)
{
  if(chdir(path))
    die("Could not change working directory to %s (%s)", path, strerror(errno));
}

#define STACK_PREFIX(X) dirs_##X
#define STACK_TYPE char *
#include <common/stack.h>

static struct dirs_stack dirs;

void dir_init(void)
{
  dirs_init(&dirs);
}

static char *dir_get(void)
{
  size_t len = DIR_LEN;
  char *buf = xmalloc(len), *result;
  while(!(result = getcwd(buf, len)) && errno == ERANGE)
    buf = xrealloc(buf, len *= 2);
  if(!result)
    die("Could not get the current working directory (%s)", strerror(errno));
  return result;
}

void dir_push(const char *path)
{
  dirs_push(&dirs, dir_get());
  dir_set(path);
}

void dir_pop(void)
{
  char *dir = dirs_pop(&dirs);
  dir_set(dir);
  xfree(dir);
}
