/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifdef DUMPER_MEMPOOL
#include "mempool_core.h"
#else
#include "memory.h"
#endif
#include <stddef.h>

#ifdef DUMPER_STRINGS
#include <cache/strdump.h>
#endif

struct DUMPER_PREFIX(block)
{
  DUMPER_TYPE data;
  DUMPER_TYPE *ptr;
};

struct DUMPER_PREFIX(state)
{
  struct DUMPER_PREFIX(block) *data;
  size_t used, size;
  void *tag;
};

#ifdef DUMPER_MEMPOOL
static void DUMPER_PREFIX(init)(struct DUMPER_PREFIX(state) *dumper, size_t size, struct mempool_core *pool)
{
  dumper->data = mempool_alloc(pool, size * sizeof *dumper->data);
#else
static void DUMPER_PREFIX(init)(struct DUMPER_PREFIX(state) *dumper, size_t size)
{
  dumper->data = xmalloc(size * sizeof *dumper->data);
#endif
  dumper->size = size;
  dumper->used = 0;
}

static void DUMPER_PREFIX(drop)(struct DUMPER_PREFIX(state) *dumper)
{
#ifndef DUMPER_MEMPOOL
  xfree(dumper->data);
#endif
}

static DUMPER_TYPE *DUMPER_PREFIX(find_internal)(struct DUMPER_PREFIX(state) *dumper, DUMPER_TYPE *ptr)
{
  for(size_t i = 0; i < dumper->used; i ++)
    if(dumper->data[i].ptr == ptr)
      return &dumper->data[i].data;
  return NULL;
}

static void DUMPER_PREFIX(push)(struct DUMPER_PREFIX(state) *dumper, DUMPER_TYPE *data)
{
  if(DUMPER_PREFIX(find_internal)(dumper, data))
    return;
  assert(dumper->used < dumper->size);
  dumper->data[dumper->used ++] = (struct DUMPER_PREFIX(block)) {
    .data = *data,
    .ptr = data
  };
#ifdef DUMPER_STRINGS
#define S(STR) dumper->data[dumper->used - 1].data.STR = cache_string_dump_push(data->STR);
  DUMPER_STRINGS
#undef S
#endif
}

static DUMPER_TYPE *DUMPER_PREFIX(find)(struct DUMPER_PREFIX(state) *dumper, DUMPER_TYPE *ptr)
{
  if(!ptr)
    return NULL;
  DUMPER_TYPE *result = DUMPER_PREFIX(find_internal)(dumper, ptr);
  return result;
}

static void DUMPER_PREFIX(resolve)(struct DUMPER_PREFIX(state) *dumper)
{
  for(size_t i = 0; i < dumper->used; i ++)
  {
#ifdef DUMPER_INTERNAL_PTRS
#define P(PTR) if(dumper->data[i].data.PTR) { dumper->data[i].data.PTR = DUMPER_PREFIX(find)(dumper, dumper->data[i].data.PTR); }
    DUMPER_INTERNAL_PTRS
#undef P
#endif
#ifdef DUMPER_EXTERNAL_PTRS
#define P(PTR, FUN) if(dumper->data[i].data.PTR) { dumper->data[i].data.PTR = FUN(dumper, dumper->data[i].data.PTR); }
    DUMPER_EXTERNAL_PTRS
#undef P
#endif
#ifdef DUMPER_STRINGS
#define S(STR) if(dumper->data[i].data.STR) { dumper->data[i].data.STR = cache_string_dump_get(dumper->data[i].data.STR); }
    DUMPER_STRINGS
#undef S
#endif
  }
}

static void DUMPER_PREFIX(check)(struct DUMPER_PREFIX(state) *dumper)
{
#ifdef DEBUG
  for(size_t i = 0; i < dumper->used; i ++)
  {
#ifdef DUMPER_INTERNAL_PTRS
#define P(PTR) assert(!dumper->data[i].data.PTR || DUMPER_PREFIX(find)(dumper, dumper->data[i].data.PTR));
    DUMPER_INTERNAL_PTRS
#undef P
#endif
  }
#endif
}

#undef DUMPER_PREFIX
#undef DUMPER_TYPE
#undef DUMPER_INTERNAL_PTRS
#undef DUMPER_EXTERNAL_PTRS
#undef DUMPER_STRINGS
#undef DUMPER_MEMPOOL
