/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#define HEAP_LIST(X) HEAP_PREFIX(list_##X)
#define LIST_PREFIX(X) HEAP_LIST(X)
#define LIST_CONTAINER HEAP_NODE
#define LIST_NODE HEAP_NODE
#define LIST_NEXT HEAP_NEXT
#define LIST_PREV HEAP_PREV
#define LIST_HEAD HEAP_HEAD
#define LIST_TAIL HEAP_TAIL
#define LIST_SIZE_VAL HEAP_SIZE
#include "list.h"

static void HEAP_PREFIX(node_clean)(HEAP_NODE *node)
{
  node->HEAP_NEXT = node->HEAP_PREV = node->HEAP_PARENT = node->HEAP_HEAD = node->HEAP_TAIL = NULL;
  node->HEAP_SIZE = 0;
  node->HEAP_COLOR = 0;
}

static void HEAP_PREFIX(clean)(HEAP_NODE *heap)
{
  HEAP_PREFIX(node_clean)(heap);
}

static void HEAP_PREFIX(insert)(HEAP_NODE *heap, HEAP_NODE *node)
{
  HEAP_PREFIX(node_clean)(node);
  if(!HEAP_LIST(size)(heap) || HEAP_LT(node, heap->HEAP_NEXT))
    heap->HEAP_NEXT = node;//Store pointer to the tree with minimum
  HEAP_LIST(append)(heap, node);
}

static void HEAP_PREFIX(merge)(HEAP_NODE *into, HEAP_NODE *what)
{
  HEAP_LIST(concat_append)(into, what);
  if(into->HEAP_NEXT && what->HEAP_NEXT && HEAP_LT(what->HEAP_NEXT, into->HEAP_NEXT))
    into->HEAP_NEXT = what->HEAP_NEXT;
}

static void HEAP_PREFIX(consolidate_max_find)(HEAP_NODE *node, unsigned *max)
{
  *max = *max > HEAP_LIST(size)(node) ? *max : HEAP_LIST(size)(node);
}

static void HEAP_PREFIX(consolidate_sort)(HEAP_NODE *node, HEAP_NODE *buckets[])
{
  unsigned size = HEAP_LIST(size)(node);
  if(!buckets[size])
  {
    buckets[size] = node;
    return;
  }
  HEAP_NODE *other = buckets[size];
  buckets[size] = NULL;
  if(HEAP_LT(other, node))
  {
    HEAP_NODE *tmp = node;
    node = other;
    other = tmp;
  }
  HEAP_LIST(append)(node, other);
  other->HEAP_PARENT = node;
  HEAP_PREFIX(consolidate_sort)(node, buckets);
}

static void HEAP_PREFIX(consolidate)(HEAP_NODE *heap)
{
  unsigned max_size = 0;
  HEAP_LIST(walk)(heap, &max_size, (HEAP_LIST(walk_func))HEAP_PREFIX(consolidate_max_find));
  max_size += HEAP_LIST(size)(heap);
  HEAP_NODE *buckets[max_size];
  for(unsigned i = 0; i < max_size; i ++)
    buckets[i] = NULL;
  HEAP_LIST(walk)(heap, buckets, (HEAP_LIST(walk_func))HEAP_PREFIX(consolidate_sort));
  HEAP_LIST(drop)(heap);
  heap->HEAP_NEXT = NULL;
  for(unsigned i = 0; i < max_size; i ++)
    if(buckets[i])
    {
      HEAP_LIST(append)(heap, buckets[i]);
      if(!heap->HEAP_NEXT || HEAP_LT(buckets[i], heap->HEAP_NEXT))
        heap->HEAP_NEXT = buckets[i];
    }
}

static void HEAP_PREFIX(parents_remove)(HEAP_NODE *node, void *data)
{
  (void) data;
  node->HEAP_PARENT = NULL;
}

static void HEAP_PREFIX(remove_root)(HEAP_NODE *heap, HEAP_NODE *root)
{
  HEAP_LIST(remove)(heap, root);
  root->HEAP_NEXT = NULL;
  HEAP_LIST(walk)(root, NULL, HEAP_PREFIX(parents_remove));
  HEAP_PREFIX(merge)(heap, root);
  HEAP_PREFIX(consolidate)(heap);
}

static HEAP_NODE *HEAP_PREFIX(remove_min)(HEAP_NODE *heap)
{
  if(!heap->HEAP_NEXT)//There are some data
    return NULL;
  HEAP_NODE *result = heap->HEAP_NEXT;
  HEAP_PREFIX(remove_root)(heap, result);
  return result;
}

static void HEAP_PREFIX(cut)(HEAP_NODE *heap, HEAP_NODE *node)
{
  HEAP_NODE *parent = node->HEAP_PARENT;
  node->HEAP_PARENT = NULL;
  node->HEAP_COLOR = 0;
  if(parent)
  {
    HEAP_LIST(remove)(parent, node);
    HEAP_LIST(append)(heap, node);
    if(parent->HEAP_COLOR)
      HEAP_PREFIX(cut)(heap, parent);
    else
      parent->HEAP_COLOR = 1;
  }
}

static void HEAP_PREFIX(remove)(HEAP_NODE *heap, HEAP_NODE *node)
{
  if(node->HEAP_PARENT)
    HEAP_PREFIX(cut)(heap, node);
  HEAP_PREFIX(remove_root)(heap, node);
}

#undef HEAP_SIZE
#undef HEAP_INSERT
#undef HEAP_CLEAN
#undef HEAP_PREFIX
#undef HEAP_NODE
#undef HEAP_NEXT
#undef HEAP_PREV
#undef HEAP_PARENT
#undef HEAP_HEAD
#undef HEAP_TAIL
#undef HEAP_COLOR
#undef HEAP_LT
#undef HEAP_LIST
