/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "file.h"
#include "debug.h"

#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#ifndef MAX_FEED_SIZE
#define MAX_FEED_SIZE 1024
#endif

int get_file(const char *filename)
{
  int result = open(filename, O_RDONLY);
  if(result < 0)
    die("Could not read file %s (%s)", filename, strerror(errno));
  return result;
}

void feed_file(const char *filename, void *data,
    feed_file_handler_t handler)
{
  int fd = get_file(filename);
  char buffer[MAX_FEED_SIZE];
  ssize_t size;
  while((size = read(fd, buffer, MAX_FEED_SIZE)) > 0)
    handler(data, buffer, size);
  if(size < 0)
    die("Error reading file %s (%s)", filename, strerror(errno));
  if(close(fd) < 0)
    die("Error closing file %s (%s)", filename, strerror(errno));
}

struct fileinfo fileinfo_get(const char *filename)
{
  struct stat buf;
  if(stat(filename, &buf))
    die("Could not stat %s (%s)", filename, strerror(errno));
  return (struct fileinfo) {
    .size = buf.st_size,
    .mod_time = buf.st_mtime
  };
}

bool file_exists(const char *filename)
{
  struct stat buf;
  if(stat(filename, &buf))
  {
    if(errno == ENOENT)
      return false;
    else
      die("Could not stat %s (%s)", filename, strerror(errno));
  }
  return true;
}
