/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef LIST_NODE_FREE
#include "memory.h"
#define LIST_NODE_FREE(node) xfree(node)
#endif

#include <stdlib.h>

#ifdef LIST_SIMPLE
#define LIST_NEXT next
#define LIST_HEAD head
#define LIST_NODE struct LIST_PREFIX(node)
#define LIST_CONTAINER struct LIST_PREFIX(t)
struct LIST_PREFIX(node)
{
  struct LIST_PREFIX(node) *next;
  LIST_SIMPLE data;
};

struct LIST_PREFIX(t)
{
  struct LIST_PREFIX(node) *head;
};
#endif

#include <assert.h>

#ifndef LIST_HEAD
#error You need head
#endif

#ifndef LIST_NEXT
#error You need next
#endif

#ifndef LIST_PREFIX
#error You need list prefix
#endif

#ifndef LIST_CONTAINER
#error Container type not specified
#endif

#ifndef LIST_NODE
#error Node type not specified
#endif

static void LIST_PREFIX(append)(LIST_CONTAINER *where, LIST_NODE *what)
{
#ifdef LIST_TAIL
#ifdef LIST_PREV
  what->LIST_PREV = where->LIST_TAIL;
#endif
  if(where->LIST_TAIL)
    where->LIST_TAIL->LIST_NEXT = what;
  else
    where->LIST_HEAD = what;
  where->LIST_TAIL = what;
#else
  LIST_NODE **last = &where->LIST_HEAD;
  while(*last)
    last = &(*last)->LIST_NEXT;
  (*last) = what;
#ifdef LIST_PREV
  what->LIST_PREV = *last;
#endif
#endif
  what->LIST_NEXT = NULL;
#ifdef LIST_SIZE_VAL
  where->LIST_SIZE_VAL ++;
#endif
}

static void LIST_PREFIX(prepend)(LIST_CONTAINER *where, LIST_NODE *what)
{
#ifdef LIST_PREV
  if(where->LIST_HEAD)
    where->LIST_HEAD->LIST_PREV = what;
  what->LIST_PREV = NULL;
#endif
  what->LIST_NEXT = where->LIST_HEAD;
  where->LIST_HEAD = what;
#ifdef LIST_TAIL
  if(!where->LIST_TAIL)
    where->LIST_TAIL = what;
#endif
#ifdef LIST_SIZE_VAL
  where->LIST_SIZE_VAL ++;
#endif
}

typedef void (*LIST_PREFIX(walk_func))(LIST_NODE *node, void *data);

static void LIST_PREFIX(walk)(LIST_CONTAINER *list, void *data,
    LIST_PREFIX(walk_func) func)
{
  LIST_NODE *head = list->LIST_HEAD, *next;
#ifdef LIST_CHECK_ENDS
  LIST_NODE *last = NULL;
  while(head && last != list->LIST_TAIL)
#else
  while(head)
#endif
  {
    next = head->LIST_NEXT;
    func(head, data);
#ifdef LIST_CHECK_ENDS
    last = head;
#endif
    head = next;
  }
}

static void LIST_PREFIX(drop)(LIST_CONTAINER *what)
{
  what->LIST_HEAD =
#ifdef LIST_TAIL
  what->LIST_TAIL =
#endif
  NULL;
#ifdef LIST_SIZE_VAL
  what->LIST_SIZE_VAL = 0;
#endif
}

static void LIST_PREFIX(clean_walk)(LIST_NODE *node, void *nothing)
{
  (void) nothing;
  LIST_NODE_FREE(node);
}

static void LIST_PREFIX(clean)(LIST_CONTAINER *list)
{
  LIST_PREFIX(walk)(list, NULL, LIST_PREFIX(clean_walk));
  LIST_PREFIX(drop)(list);
}

#ifdef LIST_SIZE_VAL
static unsigned LIST_PREFIX(size)(LIST_CONTAINER *list)
{
  return list->LIST_SIZE_VAL;
}
#else
static void LIST_PREFIX(size_walk)(LIST_NODE *nothing,
    unsigned *counter)
{
  (void) nothing;
  (*counter) ++;
}

static unsigned LIST_PREFIX(size)(LIST_CONTAINER *list)
{
  unsigned result = 0;
  LIST_PREFIX(walk)(list, &result,
      (LIST_PREFIX(walk_func)) LIST_PREFIX(size_walk));
  return result;
}
#endif

static LIST_NODE *LIST_PREFIX(last)(LIST_CONTAINER *list)
{
#ifdef LIST_TAIL
  return list->LIST_TAIL;
#else
  assert(0);
#endif
}

static LIST_NODE *LIST_PREFIX(first)(LIST_CONTAINER *list)
{
  return list->LIST_HEAD;
}

static void LIST_PREFIX(concat_append)(LIST_CONTAINER *where, LIST_CONTAINER *what)
{
  if(!what->LIST_HEAD)
    return; // Nothing to append
#ifdef LIST_TAIL
  if(where->LIST_HEAD)
  {
    where->LIST_TAIL->LIST_NEXT = what->LIST_HEAD;
#ifdef LIST_PREV
    what->LIST_HEAD->LIST_PREV = where->LIST_TAIL;
    what->LIST_HEAD = NULL;
#endif
    where->LIST_TAIL = what->LIST_TAIL;
    what->LIST_TAIL = NULL;
  }
  else
  {
    where->LIST_TAIL = what->LIST_TAIL;
    what->LIST_TAIL = NULL;
    where->LIST_HEAD = what->LIST_HEAD;
    what->LIST_HEAD = NULL;
  }
#else
  assert(0);
#endif
#ifdef LIST_SIZE_VAL
  where->LIST_SIZE_VAL += what->LIST_SIZE_VAL;
  what->LIST_SIZE_VAL = 0;
#endif
}

static LIST_NODE *LIST_PREFIX(take_first)(LIST_CONTAINER *where)
{
  LIST_NODE *node = LIST_PREFIX(first)(where);
  if(!node)
    return NULL;
  where->LIST_HEAD = node->LIST_NEXT;
#ifdef LIST_PREV
  if(where->LIST_HEAD)
    where->LIST_HEAD->LIST_PREV = NULL;
#endif
#ifdef LIST_TAIL
  if(!where->LIST_HEAD)
    where->LIST_TAIL = NULL;
#endif
#ifdef LIST_SIZE_VAL
  where->LIST_SIZE_VAL --;
#endif
  return node;
}

#ifdef LIST_PREV
static void LIST_PREFIX(remove)(LIST_CONTAINER *where, LIST_NODE *what)
{
#ifdef LIST_CHECK_ENDS
  if(what != where->LIST_HEAD)
#else
  if(what->LIST_PREV)
#endif
    what->LIST_PREV->LIST_NEXT = what->LIST_NEXT;
  else
    where->LIST_HEAD = what->LIST_NEXT;
#if defined(LIST_CHECK_ENDS) && defined(LIST_TAIL)
  if(what != where->LIST_TAIL)
#else
  if(what->LIST_NEXT)
#endif
    what->LIST_NEXT->LIST_PREV = what->LIST_PREV;
#ifdef LIST_TAIL
  else
    where->LIST_TAIL = what->LIST_PREV;
#endif
#ifdef LIST_SIZE_VAL
  where->LIST_SIZE_VAL --;
#endif
}
#else
static void LIST_PREFIX(remove)(LIST_CONTAINER *where, LIST_NODE *what)
{
  if(where->LIST_HEAD == what)
  {
    where->LIST_HEAD = what->LIST_NEXT;
#ifdef LIST_TAIL
    if(where->LIST_TAIL == what)
      where->LIST_TAIL = NULL;
#endif
  }
  else
  {
    LIST_NODE *current = where->LIST_HEAD;
#if defined(LIST_CHECK_ENDS) && defined(LIST_TAIL)
    while(current != where->LIST_TAIL
#else
    while(current
#endif
        && current->LIST_NEXT != what)
      current = current->LIST_NEXT;
    if(!current)
      return;
    current->LIST_NEXT = what->LIST_NEXT;
#ifdef LIST_TAIL
    if(where->LIST_TAIL == what)
      where->LIST_TAIL = current;
#endif
  }
#ifdef LIST_SIZE_VAL
  where->LIST_SIZE_VAL --;
#endif
}
#endif

#undef LIST_HEAD
#undef LIST_TAIL
#undef LIST_NEXT
#undef LIST_PREV
#undef LIST_CONTAINER
#undef LIST_NODE
#undef LIST_NODE_FREE
#undef LIST_SIMPLE
#undef LIST_CHECK_ENDS
#undef LIST_SIZE_VAL
#undef LIST_PREFIX
