/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "memory.h"
#include "debug.h"

#include <string.h>

#ifdef CAN_MMAP
#include <sys/mman.h>
#include <errno.h>
#endif

void *xmalloc(size_t size)
{
  void *result = malloc(size);
  if(!result)
    die("Could not malloc %zd bytes of memory\n", size);
  return result;
}

void *xmalloc_zero(size_t size)
{
  void *result = xmalloc(size);
  memset(result, 0, size);
  return result;
}

void *xrealloc(void *old, size_t size)
{
  void *result = realloc(old, size);
  if(!result && size)
    die("Could not realloc %zd bytes of memory\n", size);
  return result;
}

void xfree(void *p)
{
  free(p);
}

#ifdef CAN_MMAP
void *xmalloc_mmap(size_t size)
{
  void *result = mmap(NULL, size, PROT_READ | PROT_WRITE,
      MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
  if(result == MAP_FAILED)
    die("Could not mmap %zd bytes of memory (%s)", size,
        strerror(errno));
  dprint("Mapped %p (%zd)", result, size);
  return result;
}
#endif

void xfree_mmap(void *p, size_t size)
{
#ifdef CAN_MMAP
  dprint("Unmapping %p (%zd)", p, size);
  if(munmap(p, size))
    die("Could not unmap %zd bytes of memory from %p (%s)",
        size, (void *) p, strerror(errno));
#else
  xfree(p);
#endif
}

void *xrealloc_mmap(void *p, size_t old_size, size_t new_size)
{
#ifdef CAN_MMAP
  void *result = mremap(p, old_size, new_size, MREMAP_MAYMOVE);
  if(result == MAP_FAILED)
    die("Could not remap %zd->%zd on %p (%s)", old_size,
        new_size, p, strerror(errno));
#else
  void *result = realloc(p, new_size);
  if(!result && new_size)
    die("Could not realloc %zd->%zd on %p", old_size,
        new_size, p);
#endif
  return result;
}
