/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "memory.h"
#include "mempool_core.h"

#include <stdbool.h>

#ifdef MEMPOOL_WRAP

struct MEMPOOL_PREFIX(t)
{
  struct mempool_core *core;
  bool active;
};

static void MEMPOOL_PREFIX(init)(struct MEMPOOL_PREFIX(t) *mempool, struct mempool_core *core)
{
  *mempool = (struct MEMPOOL_PREFIX(t)) {
    .core = core,
    .active = false
  };
}

static void MEMPOOL_PREFIX(clean)(struct MEMPOOL_PREFIX(t) *pool)
{
  if(pool->active && !--pool->core->refs)
    mempool_core_clean(pool->core);
  pool->active = false;
}

static void *MEMPOOL_PREFIX(alloc_internal)(struct MEMPOOL_PREFIX(t) *pool, size_t size)
{
  if(!pool->active)
  {
    pool->active = true;
    pool->core->refs ++;
  }
  return mempool_alloc(pool->core, size);
}

#define MEMPOOL_CONTAINER struct MEMPOOL_PREFIX(t)
#define MEMPOOL_ALLOC(POOL, SIZE) MEMPOOL_PREFIX(alloc_internal)

#else

#define MEMPOOL_CONTAINER struct mempool_core
#define MEMPOOL_ALLOC mempool_alloc

#endif

#ifdef MEMPOOL_HARDCODE_VARIABLE

#define MEMPOOL_VAR MEMPOOL_HARDCODE_VARIABLE
#define MEMPOOL_PARAM
#define MEMPOOL_PARAM_ONLY void

#else

#define MEMPOOL_VAR pool
#define MEMPOOL_PARAM_ONLY MEMPOOL_CONTAINER *pool
#define MEMPOOL_PARAM MEMPOOL_PARAM_ONLY,

#endif

static MEMPOOL_TYPE *MEMPOOL_PREFIX(init_array)(MEMPOOL_TYPE *data, size_t count)
{
#ifdef MEMPOOL_INIT
  for(size_t i = 0; i < count; i ++)
  {
    MEMPOOL_INIT(data + i);
  }
#endif
  return data;
}

static MEMPOOL_TYPE *MEMPOOL_PREFIX(get)(MEMPOOL_PARAM_ONLY)
{
  return MEMPOOL_PREFIX(init_array)(MEMPOOL_ALLOC(MEMPOOL_VAR, sizeof(MEMPOOL_TYPE)), 1);
}

static MEMPOOL_TYPE *MEMPOOL_PREFIX(get_array)(MEMPOOL_PARAM size_t count)
{
  return MEMPOOL_PREFIX(init_array)(MEMPOOL_ALLOC(MEMPOOL_VAR, count * sizeof(MEMPOOL_TYPE)), count);
}

#undef MEMPOOL_PREFIX
#undef MEMPOOL_TYPE
#undef MEMPOOL_INIT
#undef MEMPOOL_VAR
#undef MEMPOOL_PARAM
#undef MEMPOOL_PARAM_ONLY
#undef MEMPOOL_ALLOC
#undef MEMPOOL_WRAP
#undef MEMPOOL_HARDCODE_VARIABLE
