/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef COMMON_MEMPOOL_CORE_H
#define COMMON_MEMPOOL_CORE_H

#include <stddef.h>
#include <stdbool.h>

struct pool_page;

struct mempool_core
{
  size_t align;
  struct pool_page *first;
  unsigned char *data;
  size_t empty;
  size_t references;
  size_t borrowed;
  struct restart_cleanup *cleanup;
  bool auto_cleanup:1;
};

// If called with align = 0, default CPU allignment is used
void mempool_core_init(struct mempool_core *mcore, size_t align, bool auto_cleanup);
void mempool_core_clean(struct mempool_core *mcore);
// Memory zeroed
void *mempool_alloc(struct mempool_core *mcore, size_t size) __attribute__((malloc));
// Memory not zeroed
void *mempool_alloc_resizeable(struct mempool_core *mcore, size_t *size) __attribute__((malloc));
void *mempool_grow(struct mempool_core *mcore, void *data, size_t *new_size);
void *mempool_size_set(struct mempool_core *mcore, void *data, size_t size);
// Memory not zeroed
// Ensures the buffer has at last size bytes, but it is reused at the
// next allocation - it is valid until the mempool is used again.
void *mempool_borrow(struct mempool_core *mcore, size_t size);
void *mempool_alloc_nozero(struct mempool_core *mcore, size_t size);

extern struct mempool_core persistent_pool;

static inline struct mempool_core *pool_choose(struct mempool_core *tmp_pool, bool temporary)
{
  return temporary ? tmp_pool : &persistent_pool;
}

void mempool_src_init(void);

#endif
