/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "memory.h"

#include <stdlib.h>
#include <assert.h>

#define STACK_STRUCT struct STACK_PREFIX(stack)
#ifndef STACK_BASE_SIZE
#define STACK_BASE_SIZE 10
#endif
#ifndef STACK_GROW
#define STACK_GROW 2
#endif

STACK_STRUCT
{
  STACK_TYPE *data;
  size_t size, alloc;
};

static void STACK_PREFIX(init)(STACK_STRUCT *stack)
{
  stack->data = xmalloc((STACK_BASE_SIZE) * sizeof *stack->data);
  stack->alloc = STACK_BASE_SIZE;
  stack->size = 0;
}

static void STACK_PREFIX(free)(STACK_STRUCT *stack)
{
  xfree(stack->data);
}

static size_t STACK_PREFIX(size)(STACK_STRUCT *stack)
{
  return stack->size;
}

static STACK_TYPE STACK_PREFIX(top)(STACK_STRUCT *stack)
{
  assert(stack->size);
  return stack->data[stack->size - 1];
}

static void STACK_PREFIX(push)(STACK_STRUCT *stack, STACK_TYPE value)
{
  if(stack->size == stack->alloc)
    stack->data = xrealloc(stack->data, (stack->alloc *= (STACK_GROW)) *
        sizeof *stack->data);
  stack->data[stack->size ++] = value;
}

static STACK_TYPE STACK_PREFIX(pop)(STACK_STRUCT *stack)
{
  assert(stack->size);
  return stack->data[-- stack->size];
}

static void STACK_PREFIX(clean)(STACK_STRUCT *stack)
{
  stack->data = xrealloc(stack->data, (stack->alloc = STACK_BASE_SIZE) *
      sizeof *stack->data);
  stack->size = 0;
}

#undef STACK_TYPE
#undef STACK_PREFIX
#undef STACK_BASE_SIZE
#undef STACK_GROW
