/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "stringpool.h"
#include "mempool_core.h"

#include <assert.h>
#include <string.h>

struct mempool_core strpool;

void strpool_src_init(void)
{
  strpool_init(&strpool);
}

char *strpool_str_get(struct mempool_core *pool, size_t size)
{
  char *result = mempool_alloc(pool, size + 1);
  result[size] = '\0';
  return result;
}

char *strpool_dupe(struct mempool_core *pool, const char *string,
    size_t len)
{
  char *result = strpool_str_get(pool, len);
  memcpy(result, string, len);
  return result;
}

char *strpool_dup(struct mempool_core *pool, const char *string)
{
  return string ? strpool_dupe(pool, string, strlen(string)) : NULL;
}

char *strpool_dupn(struct mempool_core *pool, const char *string,
    size_t max)
{
  size_t len = strlen(string);
  if(len > max)
    len = max;
  return strpool_dupe(pool, string, len);
}

void strpool_init(struct mempool_core *pool)
{
  mempool_core_init(pool, 1, true);
}

static char *strpool_concat_internal(struct mempool_core *pool, void *(*alloc)(struct mempool_core *, size_t),
    const char *first, const char *second)
{
  size_t first_len = strlen(first);
  size_t second_len = strlen(second);
  size_t len = first_len + second_len;
  char *result = alloc(pool, len + 1);
  if(result != first)
    memcpy(result, first, first_len);
  memcpy(result + first_len, second, second_len);
  result[len] = '\0';
  return result;
}

char *strpool_concat(struct mempool_core *pool, const char *first, const char *second)
{
  return strpool_concat_internal(pool, mempool_alloc_nozero, first, second);
}

char *strpool_concat_borrow(struct mempool_core *pool, const char *first, const char *second)
{
  return strpool_concat_internal(pool, mempool_borrow, first, second);
}
