/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "external.h"
#include "flags.h"
#include "coroutine.h"

#include <common/dir.h>
#include <common/debug.h>

#include <unistd.h>
#include <string.h>
#include <errno.h>

pid_t external_run(const char *command[], int *outpipe, const char *workdir)
{
  int pipes[2];
  if(outpipe)
  {
    if(pipe(pipes))
      die("Could not create pipe (%s)", strerror(errno));
    *outpipe = pipes[0];
  }
  pid_t child = fork();
  if(child == -1)
    die("Could not fork (%s)", strerror(errno));
  if(child)
  {
    if(outpipe)
    {
      if(close(pipes[1]))
        die("Could not close pipe (%s)", strerror(errno));
      fd_nonblock(*outpipe);
    }
    return child;
  }
  else
  {
    coroutine_preexec();
    if(outpipe)
    {
      if(dup2(pipes[1], 1) == -1)
        die("Could not redirect stdout to %d (%s)", pipes[1], strerror(errno));
      if(close(pipes[0]) || close(pipes[1]))
        die("Could not close pipe (%s)", strerror(errno));
    }
    if(workdir)
      dir_set(workdir);
    execv(command[0], (char *const *) command);
    // If I'm here, something is wrong
    die("Could not exec %s (%s)", command[0], strerror(errno));
  }
}
