/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "shell.h"
#include "coroutine.h"

#include <lang/var.h>
#include <common/debug.h>

#include <string.h>

static char *escape_buff;
size_t escape_buff_len;

static size_t block_push(size_t pos, const char *data, size_t length)
{
  if(pos + length > escape_buff_len)
  {
    size_t new_size = pos + length;
    if(new_size < escape_buff_len * 2)
      new_size = escape_buff_len * 2;
    escape_buff = realloc(escape_buff, escape_buff_len = new_size);
  }
  memcpy(escape_buff + pos, data, length);
  return pos + length;
}

char *array_escape(struct variable *array)
{
  size_t pos = 0;
  for(size_t i = 0; i < array->count; i ++)
  {
    if(i)
      pos = block_push(pos, " ", 1);
    switch(array->values[i].type)
    {
      case vt_string:
/*        pos = block_push(pos, array->values[i].string,
            strlen(array->values[i].string));
        break;*/
      case vt_object:
      {
        pos = block_push(pos, "'", 1);
        const char *apos = array->values[i].string, *last = apos;
        while(apos = index(apos, '\''))
        {
          pos = block_push(pos, last, apos - last);
          last = apos + 1;
          pos = block_push(pos, "'\\''", 4);
        }
        pos = block_push(pos, last, strlen(last));
        pos = block_push(pos, "'", 1);
        break;
      }
      default:
        diee("Trying to stringify class, function or method");
    }
  }
  block_push(pos, "", 1);
  return escape_buff;
}

char *shell_exec(const char *command, bool capture, struct mempool_core *pool, const char *workdir)
{
  dprint("Executing shell command %s", command);
  const char *args[] = {
    "/bin/sh",
    "-c",
    command,
    NULL
  };
  int result;
  char *text = coroutine_run_command(args, capture, pool, &result, workdir);
  if(result)
    die("Command \"%s\" terminated with %d error status\n",
        command, result);
  return text;
}
