/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "var.h"

#ifndef LANG_CONTEXT_H
#define LANG_CONTEXT_H

#include <stdbool.h>

struct block_scan_result;
struct lang_node;

#define TRIE_PREFIX(X) var_trie_##X
#define TRIE_DATA struct var_holder *
#define TRIE_DATA_DEFAULT NULL
#include <common/trie.h>

struct mempool_core;
struct object;
struct context;

struct context_plug
{
  struct context_plug *next;
  struct context *context;
};

struct context
{
  struct var_trie_t variables;
  struct block_scan_result *block;
  struct mempool_core *context_pool; // For variables, etc
  struct object *object; // Object the context belongs to, or NULL
  struct context *parent_of;  // For object inheritance, so it can be searched for deps setting
  struct context_plug parents; // List of parents
  size_t cache_refs;
  struct var_dep_list *dep_list;
  bool function:1;      // For a different searching & setting of variables
  bool builtin:1;
  bool cache_scheduled:1;
  bool cache_valid:1;
  // For storage in cache
  struct context *cache_next, *cache_prev;
  size_t context_num;
  const char *filename;
  unsigned token;
  struct lang_node *node;
};

#define LIST_PREFIX(X) plug_list_##X
#define LIST_CONTAINER struct context_plug
#define LIST_NODE struct context_plug
#define LIST_HEAD next
#define LIST_NEXT next
#include <common/list.h>

void lang_context_init(struct mempool_core *tmp_pool);
void lang_context_restart(void);
struct context *context_get(struct mempool_core *pool);
void context_return(struct context *context);
void context_prepare(struct context *context);

struct var_holder *context_var_search(struct context *context, const char *name, bool parents);
struct var_holder *context_var_set(struct context *context, const char *name, struct variable *value, bool overwrite,
    bool subscripted);
void context_var_reference(struct context *context, const char *name, struct var_holder *holder, bool overwrite);
void context_raw_block(struct block_scan_result *result, struct lang_node *node);
void context_export_func(struct context *context);

// Note: it can have multiple parents, can be used more than once
// They apply in the reverse order they were plugged
// (the last the first to search)
void contexts_plug(struct context *parent, struct context *child);
struct context *context_parent_first(struct context *context);
void context_var_set_global(struct context *context, const char *name);
void context_toplevel_load(struct mempool_core *tmp_pool);

extern struct context *context_toplevel;

#endif
