/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef LANG_FUNCTION_H
#define LANG_FUNCTION_H

#include <stddef.h>
#include <stdbool.h>

struct function;
struct lang_node;
struct variable;
struct mempool_core;
struct context;

enum param_type
{
  pt_single,
  pt_multi,
  pt_stop   // For buildins with arbitrary number of params
};

struct param
{
  enum param_type type;
  size_t count;
  struct var_value *values;
};

typedef struct variable *(*func_code)(struct variable **params, size_t count, struct context *context,
    struct context *parent, struct mempool_core *tmp_pool, bool temporary, struct lang_node *lang_node,
    struct function *function, bool object_toplevel);

struct function
{
  int pcount;            // Param count
  func_code code;
  size_t cache_id;
  struct lang_node *block, *func, *note;
  struct variable *compiled;
  struct context *func_context;
  struct param_def {
    const char *name;
    enum param_type type;
  } params[];
};

void function_build(struct lang_node *node, struct context *context, struct variable *compiled,
    struct mempool_core *tmp_pool, bool temporary, struct var_value *value);

struct variable *function_apply(struct function *function, size_t param_count, struct variable **params,
    struct context *context, struct context *parent, struct mempool_core *tmp_pool, bool temporary,
    struct lang_node *lang_node, bool object_toplevel);

struct variable *function_user_eval(struct variable **params, size_t count, struct context *context,
    struct context *parent, struct mempool_core *tmp_pool, bool temporary, struct lang_node *lang_node,
    struct function *function, bool object_toplevel);

#endif
