/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "node.h"

#include <common/debug.h>

#define RECYCLER_PREFIX(X) lang_node_recycler_##X
#define RECYCLER_TYPE struct lang_node
#define RECYCLER_NEXT next
#include <common/recycler.h>

#include <stdio.h>
#include <string.h>

struct lang_node_recycler_t lang_node_recycler;

void lang_node_init(void)
{
  lang_node_recycler_init(&lang_node_recycler, &persistent_pool);
}

static void node_print_rec(struct lang_node *node, unsigned *depth)
{
  for(unsigned i = 0; i < *depth; i ++)
    fputc(' ', stderr);
  fprintf(stderr, "'%s': %u\n", node->data, (unsigned) node->type);
  unsigned num = *depth + 1;
  langlist_walk(node, &num, (langlist_walk_func) node_print_rec);
}

void lang_tree_print(struct lang_node *node)
{
  unsigned zero = 0;
  node_print_rec(node, &zero);
}

struct lang_node *lang_node_alloc(void)
{
  struct lang_node *node = lang_node_recycler_get(&lang_node_recycler);
  memset(node, 0, sizeof *node);
  return node;
}

void lang_node_free(struct lang_node *node)
{
  dprint("Freeing node %d:%s", node->type, node->data);
  lang_node_recycler_put(&lang_node_recycler, node);
}

void lang_node_restart(void)
{
  lang_node_recycler_clean(&lang_node_recycler);
}
