/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef LANG_NODE_H
#define LANG_NODE_H

#include <stdbool.h>

enum lang_node_type
{
  ln_none = 0, //Internal
  ln_block,
  ln_string,
  ln_variable,
  ln_variable_escape,
  ln_string_concat,
  ln_perl,
  ln_subscript,
  ln_method,
  ln_array_concat,
  ln_shell,
  ln_assign_direct,
  ln_assign_alter,
  ln_inherit_direct,
  ln_inherit_alter,
  ln_eval,
  ln_parent,
  ln_descend,
  ln_import,
  ln_inherited,
  ln_export,
  ln_global,
  ln_nocache,
  ln_fun_def,
  ln_class_def,
  ln_function,
  ln_param_multi,
  ln_param_single,
  ln_map,
  ln_extract
};

struct block_scan_result;

struct lang_node
{
  enum lang_node_type type;
  unsigned short priority;
  struct lang_node *children_head, *children_tail, *next;
  const char *data;
  unsigned line, oper, token;
  const char *filename;
  struct block_scan_result *block_scan;
};

struct lang_node *lang_node_alloc(void);
void lang_node_free(struct lang_node *node);
void lang_node_init(void);
void lang_node_restart(void);

void lang_tree_print(struct lang_node *node);

struct node_bind
{
  struct node_bind *next;
  struct lang_node *node;
  bool absolute:1;        // For assignment nodes
};

struct node_bind_container
{
  struct node_bind *head, *tail;
};

#define LIST_PREFIX(X) node_bind_##X
#define LIST_CONTAINER struct node_bind_container
#define LIST_NODE struct node_bind
#define LIST_NEXT next
#define LIST_HEAD head
#define LIST_TAIL tail
//#define LIST_CHECK_ENDS
#include <common/list.h>

#define LIST_PREFIX(X) langlist_##X
#define LIST_CONTAINER struct lang_node
#define LIST_NODE struct lang_node
#define LIST_HEAD children_head
#define LIST_TAIL children_tail
#define LIST_NEXT next
#include <common/list.h>

#endif
