/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef LANG_VAR_H
#define LANG_VAR_H

#include "function.h"
#include "node.h"
#include <stddef.h>
#include <stdbool.h>

struct block;
struct mempool_core;
struct object;

enum var_state
{
  vs_untouched,
  vs_processing,
  vs_computed,
  vs_cache
};

enum value_type
{
  vt_string,
  vt_object,
  vt_function,
  vt_method,
  vt_class
};

// Single variable value
struct var_value
{
  enum value_type type:3;
  bool str_temporary:1;
  bool fun_temporary:1;
  const char *string;
  struct object *object;
  struct function *function;
};

void lang_var_init(void);
void lang_var_restart(void);

struct var_dep_list;

// A variable with multiple values
struct variable
{
  enum var_state state:2;
  bool temporary:1;
  bool undefined:1;
  size_t cache_pos;
  size_t count;
  unsigned cache_id;
  struct var_dep_list *deps;
  struct var_value values[];
};

extern struct variable var_empty, var_undef;

/*
 * Dependencies for variables - on input files.
 * TODO What about evals and such?
 */
struct var_dep;
struct var_dep_list
{
  struct var_dep *head, *tail;
  struct var_dep_list *next;
  struct var_dep_list *next_filter;
  struct var_dep *next_test;
  bool changed_resolved:1;
  bool changed:1;
  size_t cache_number;
  struct variable var_empty;
  struct variable var_undef;
};

struct var_dep_list *var_dep_file(const char *filename);
struct var_dep_list *var_dep_lists_merge(struct var_dep_list *_1, struct var_dep_list *_2);

/*
 * Allocates a variable with this number of values.
 * Sets its state to processing.
 */
struct variable *var_alloc(struct mempool_core *tmp_pool, size_t val_count, bool temporary);

struct variable *var_make_persistent(struct variable *variable);
void var_values_make_persistent(struct variable *variable);
bool var_eq(struct variable *_1, struct variable *_2);

#include "context.h"

struct var_holder
{
  struct node_bind_container creators;
  struct variable *variable;
  size_t ref_count;
  struct var_holder *next;
  bool subscripted:1;
  bool global:1;
  bool exported:1;
};

#endif
