/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <common/debug.h>
#include <common/common.h>
#include <common/config.h>
#include <common/restart.h>
#include <parser/parser.h>
#include <cache/cache.h>
#include <lang/lang.h>
#include <lang/context.h>
#include <lang/compute.h>
#include <hierarchy/hierarchy.h>
#include <plan/plan.h>
#include <interact/interact.h>

static struct mempool_core tmp_pool;

int main(int argc, char *argv[])
{
  gdb_catch_signals();
  mempool_core_init(&tmp_pool, 0, true);
  common_init(argc, argv, &tmp_pool);
  restart_init();
  cache_init();
  parser_init();
  lang_init(&tmp_pool);
  hierarchy_build();
  restart_finish();
  interact_init();
  // TODO Resolve subscripts
  struct variable *targets = variable_compute(context_top, config_targets, NULL);
  if(targets->undefined)
    diee("TARGETS variable is not defined");
  dprinte("List of targets:");
  for(size_t i = 0; i < targets->count; i ++)
  {
    if(targets->values[i].type != vt_object)
      diee("Target is not an object");
    dprint("%s", targets->values[i].string);
  }
  plan_init();
  plan_add(targets);
  plan_run(&tmp_pool);
  cache_store(cache_filename);
  return 0;
}
