/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "shape.h"

#include <lang/node.h>
#include <common/debug.h>
#include <common/config.h>

#include <string.h>
#include <stdbool.h>

static void node_compact_walk(struct lang_node *node, struct lang_node *dest)
{
  if(node->type == dest->type)
  {
    dprinte("Compacting");
    langlist_concat_append(dest, node);
    lang_node_free(node);
  }
  else
    langlist_append(dest, node);
}

static void node_compact(struct lang_node *node)
{
  dprint("Trying to compact node %s (line %u, type %u)", node->data, node->line, (unsigned) node->type);
  struct lang_node copy = *node;
  langlist_drop(node);
  langlist_walk(&copy, node, (langlist_walk_func) node_compact_walk);
}

static void parenth_compact(struct lang_node *node)
{
  dprinte("Turning parenthesis into an array");
  node->type = ln_array_concat;// Make it an array
}

static void tree_postprocess_walk(struct lang_node *node, struct lang_node *parent)
{
  struct lang_node copy = *node;
  langlist_drop(node);
  bool drop = false;
  langlist_walk(&copy, node, (langlist_walk_func) tree_postprocess_walk);
  if(node->type == ln_function && !node->data) // Just parenthesis - make an array
    parenth_compact(node);
  if(node->type == ln_array_concat && parent && (parent->type == ln_inherit_direct || parent->type == ln_inherit_alter))
  { // Remove arrays in inheritance - they are not needed
    node_compact(node);
    langlist_concat_append(parent, node);
    drop = true;
  }
  else if(node->type == ln_string_concat || node->type == ln_array_concat)
  {
    node_compact(node);
    if(langlist_size(node) == 1)
    {
      dprinte("Array or string concat with single node");
      if(parent)
        langlist_append(parent, langlist_first(node));
      drop = true;
    }
  }
  if(!drop && parent)
    langlist_append(parent, node);
  else if(drop)
    lang_node_free(node);
}

static void tree_inheritance_walk(struct lang_node *node, struct lang_node *nodes[])
{
  if(node->type == ln_inherit_direct || node->type == ln_inherit_alter)
  {
    dprinte("Inheritance correction");
    struct lang_node *owner = langlist_take_first(node), copy = *node, cache, *new_nodes[] = {
      node,
      nodes[1] ? nodes[1] : owner,
      nodes[1] ? nodes[2] : &cache,
      nodes[1] ? nodes[3] : node
    };
    langlist_append(nodes[0], owner);
    langlist_append(new_nodes[1], node);
    langlist_drop(node);
    langlist_drop(&cache);
    langlist_walk(&copy, new_nodes, (langlist_walk_func) tree_inheritance_walk);
    langlist_concat_append(owner, &cache);
  }
  else
  {
    if(nodes[0])
      langlist_append(nodes[2 * (node->type == ln_block && nodes[1])], node);
    struct lang_node copy = *node, *new_nodes[] = { node, NULL };
    langlist_drop(node);
    langlist_walk(&copy, new_nodes, (langlist_walk_func) tree_inheritance_walk);
  }
}

struct lang_node *tree_postprocess(struct lang_node *node)
{
  tree_postprocess_walk(node, NULL);
  struct lang_node *nodes[] = { NULL, NULL }; /* Parent, Owner, Cache (if Owner present), Last inheritange (if Owner present) */
  tree_inheritance_walk(node, nodes);
  dprinte("Postprocessed tree:");
#ifdef TRACE
  if(config_debug)
    lang_tree_print(node);
#endif
  return node;
}
