/*
        Remake
        Copyright (C) 2008-2009 Michal Vaner (vorner@ucw.cz)

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        he Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef PARSER_TOKEN_H
#define PARSER_TOKEN_H

#include <stdlib.h>

struct token_state;

// Types of reported tokens
enum token_type
{
  tt_string_start,
  tt_string_data,
  tt_string_var,
  tt_string_var_escaped,
  tt_string_end,

  tt_command_end,

  tt_ident,
  tt_perl,

  tt_vartype,

  tt_comma,
  tt_operator,

  tt_start_subscript,
  tt_end_subscript,
  tt_open_paren,
  tt_close_paren,
  tt_open_block,
  tt_close_block,

  tt_internal
};

// Types of handler functions
typedef void (*token_error_handler_t)(void *data, const char *msg);
typedef void (*token_handler_t)(void *data, const char *token,
    size_t len, enum token_type type, unsigned line);

/*
 * Creates a new tokenizer.
 * Pass two handlers (for errors and tokens) and some your data.
 * You can provide NULL for any of these.
 */
struct token_state *token_create(void *data,
    token_error_handler_t error_handler, token_handler_t handler);
/*
 * Parse some more data.
 */
void token_process(struct token_state *state, const char *buffer,
    size_t len);
/*
 * First, pushes trough all yet unparsed parts.
 * Then destroys the tokenizer.
 *
 * Returns number of errors encountered during parsing.
 */
unsigned token_destroy(struct token_state *state);

#endif //PARSER_TOKEN_H
